/*
 * Expresso, a C++ Array template class library
 * Copyright (C) 1998 Oskar Enoksson (enok@lysator.liu.se)
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#include <iostream>

#include "expresso/flex.h"
#include "expresso/fix.h"
#include "expresso/standardop.h"

#include <time.h>
#include <stdlib.h>

using expresso::Array;
using expresso::Fix;
using expresso::Ix;
using namespace std;

int main() {
   clock_t time1,time2;
   int nmax;
   int jmax,kmax;
   const int imax=3;
   long flops;
   int i,j,k;

   cout<<"Enter #Mflop to run ";
   cin>>flops;
   flops*=1000000;
   cout<<"Enter two index limits ";
   cin>>jmax>>kmax;

   Array<3,Fix<float> > q(imax,jmax,kmax);
   Array<3,Fix<float> > a(imax,jmax,kmax);
   Array<3,float> b(imax,jmax,kmax);

   for(k=q.L(2); k<q.U(2); k++)
     for(j=q.L(1); j<q.U(1); j++)
       for(i=q.L(0); i<q.U(0); i++)
	 q(i,j,k)=(float)rand()/RAND_MAX;

   nmax=(int)((float)flops/(5*imax*jmax*kmax));

   cout<<"Testing fix expression"<<endl;
   a<<=q;
   time1=clock();
   for(int n=0; n<nmax; n++)
     a<<=2*a-.5f*a*a*a;

   time2=clock();

   cout<<flops/(float(time2-time1)/CLOCKS_PER_SEC)/1000000<<" MFlops"<<endl;
   cout<<*a.Start()<<endl;

   cout<<"Testing fix indexing"<<endl;
   a<<=q;
   time1=clock();
   for(int n=0; n<nmax; n++) {
     for(int k=kmax-1; k>=0; k--)
       for(int j=jmax-1; j>=0; j--)
	 for(int i=imax-1; i>=0; i--)
	   a(i,j,k)=2*a(i,j,k)-.5f*a(i,j,k)*a(i,j,k)*a(i,j,k);
   }
   time2=clock();

   cout<<flops/(float(time2-time1)/CLOCKS_PER_SEC)/1000000<<" MFlops"<<endl;
   cout<<*a.Start()<<endl;

   cout<<"Testing flex expression"<<endl;
   b<<=q;
   time1=clock();
   for(int n=0; n<nmax; n++)
     b<<=2*b-.5f*b*b*b;

   time2=clock();

   cout<<flops/(float(time2-time1)/CLOCKS_PER_SEC)/1000000<<" MFlops"<<endl;
   cout<<*b.Start()<<endl;
   
   cout<<"Testing flex indexing"<<endl;
   b<<=q;
   time1=clock();
   for(int n=0; n<nmax; n++) {
     for(int k=kmax-1; k>=0; k--)
       for(int j=jmax-1; j>=0; j--)
	 for(int i=imax-1; i>=0; i--)
	   b(i,j,k)=2*b(i,j,k)-.5f*b(i,j,k)*b(i,j,k)*b(i,j,k);
   }
   time2=clock();

   cout<<flops/(float(time2-time1)/CLOCKS_PER_SEC)/1000000<<" MFlops"<<endl;
   cout<<*b.Start()<<endl;
}
