/*
 * Expresso, a C++ Array template class library
 * Copyright (C) 1998 Oskar Enoksson (enok@lysator.liu.se)
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
//
// The following code solves Laplaces equation using
// finite differences in a very simple and inefficient way,
// just for an example of what can be done
//

#include <iostream>
#include <cmath>
#include "expresso.h"

#define NX 500
#define NY 500
#define DX 0.25
#define PI 3.141592654

using namespace std;
using namespace esso;

int main() {
// Declare arrays x (state) and res (residual)
   Array<2,Fix<float> > x(NX,NY);
   Array<2,Fix<float> > res(NX-1,NY-1);
   float err=1;
   int i,j;
   
// Set x to initial value zero
   x<<=0;

// Set boundary values
   for(i=1; i<NX-1; i++) {
     x(i,1)=sin(2*PI*i/NX);
     x(i,NY-1)=-sin(2*PI*i/NX);
   }
   for(j=1; j<NY-1; j++) {
     x(1,j)=sin(2*PI*j/NY);
     x(NX-1,j)=-sin(2*PI*j/NY);
   }

// Begin iteration loop
   while(err>0.001) {
      err=0;
// Calculate residual
      for(j=1; j<NY-1; j++)
	for(i=1; i<NX-1; i++) {
	   res(i,j)=x(i+1,j)+x(i-1,j)+x(i,j+1)+x(i,j-1)-4*x(i,j);
// Sum up error
	   err=err+res(i,j)*res(i,j);
	}
      cout<<err<<endl;
      for(j=1; j<NY-1; j++)
	for(i=1; i<NX-1; i++)
	  x(i,j)=x(i,j)+DX*res(i,j);
   }
}
