/*
 * Expresso, a C++ Array template class library
 * Copyright (C) 1998 Oskar Enoksson (enok@lysator.liu.se)
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

//
// File purpose: Example of using Array<N,T> objects
//

#include <iostream>
#include "expresso.h"

using namespace esso;
using namespace std;

template<class Type, class constType>
class Foo {
public:
  static void test();
};

template<class Type, class constType>
void Foo<Type, constType>::test() {
  Array<2,Type> a1;     

  Array<2,Type> a2(15,25);

  Ix<2> len(10,20);
  Array<2,Type> a3(len);

  Ix<2> bottom(1,5);
  Array<2,Type> a4(len,bottom);

  Array<2,Type> a5(a4);

  Array<2,Type> a6 = a4;

  Array<2,constType> a7 = a4;

  for(int n=0; n<2; n++)
    cout << a4.L(n) <<' '<<a4.U(n)<<' '<<a4.Len(n)<<' '<<a4.Stride(n)<<endl;

  cout<<(void *)a4.Start()<<endl;

  cout<<(void *)a4.Origo()<<endl;

  a4.Data();

  cout<<a4.IsNull()<<endl;
  
  a4(6,8)=3;
  cout<<a4(6,8)<<endl;

  cout<<a4[Ix<2>(6,8)]<<endl;

  a4.Dealloc();

  a4.Realloc(len);

  a4.Realloc(len, bottom);

  Array<1,Type> a8((Ix<1>)(len[0]*len[1]));
  a4.Realloc(len, bottom, a8.Data(), a8.Origo()-bottom[0]-bottom[1]*len[0]);

  a3.Realloc(a4);

  a3.Refer(a4);
  a3=a4;

  a3.Clone(a4);

  a3.Clone();

  a4.Shift(0,15);

  a4.Shift(bottom);

  a4.Restrict(1, 4, 10);

  a4.Restrict(Ix<2>(2,4), Ix<2>(8,10));

  a4.RestrictL(0, 2);
  a4.RestrictL(Ix<2>(2,4));
  a4.RestrictU(0,8);
  a4.RestrictU(Ix<2>(8,10));
}

int main() {
  Foo<float,const float>::test();
  Foo<Fix<int>, Fix<const int> >::test();
}
